/*
 * Copyright 1990 MCNC and Vivek Khera
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of MCNC not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  MCNC makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * MCNC DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL MCNC
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:  Vivek Khera, MCNC
 */

/*
 * xuser is an interactive interface to the KACL extension for adding and
 * deleting user names from the list maintained by the extension. it is based
 * on loosely xhost, and the command line interface is the same. 
 *
 * V. Khera	31-JUL-1990 
 */

static char     rcsId[] = "$Header: xuser.c,v 1.4 90/09/18 10:29:43 bin Exp $";

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/extensions/Kacl.h>

static int local_xerror();
static char *ProgramName;

main(argc, argv)
int             argc;
char          **argv;
{
	register char *arg;
	int i;
	Display *dpy;

	ProgramName = argv[0];
	if ((dpy = XOpenDisplay(NULL)) == NULL) {
		fprintf(stderr, "%s:  unable to open display \"%s\"\n",
			ProgramName, XDisplayName(NULL));
		exit(1);
	}

	XSetErrorHandler(local_xerror);

	if (argc == 1) {
		register char *list = XKACLGetUsers(dpy);
		register char *next = list;
		char *p = list;	/* save for deleting later */
		int done;

		if (list)
			printf("The following names are in the ACL:\n");
		else {
			printf("The ACL is empty\n");
			exit(0);
		}
		/* massage colon-separated list and print it */
		done = (*next == '\0');
		while (!done) {
			while (*next != ':' && *next != '\0')
				++next;
			done = (*next == '\0');	/* last one already null */
			*next++ = '\0';	/* null terminate this part of list */
			printf(" %s\n", list);
			list = next;
		}
		XKACLFreeList(p);			
		exit(0);
	}
	if (argc == 2 && !strcmp(argv[1], "-help")) {
		fprintf(stderr, "usage: %s [[+-]user ...]\n", ProgramName);
		exit(1);
	}
	for (i = 1; i < argc; i++) {
		arg = argv[i];
		if (*arg == '-') {

			if (!argv[i][1] && ((i + 1) == argc)) {
				fprintf(stderr, "no name given\n");
			} else {
				arg = argv[i][1] ? &argv[i][1] : argv[++i];
				XKACLDeleteUser(dpy, arg);
			}
		} else {
			if (*arg == '+' && !argv[i][1] && ((i + 1) == argc)) {
				fprintf(stderr, "no name given\n");
			} else {
				if (*arg == '+') {
					arg = argv[i][1] ? &argv[i][1] : argv[++i];
				}
				XKACLAddUser(dpy, arg);
			}
		}
	}
	XCloseDisplay(dpy);	/* does an XSync first */
	exit(0);
}

/* 
 * local_xerror() handles local non-fatal errors.  the only one 
 * currently defined is that the add/delete functions may return 
 * BadAccess if the user is not on the local host.
 */
static int local_xerror(dpy, rep)
Display *dpy;
XErrorEvent *rep;
{
	if (rep->error_code == BadAccess) {
		fprintf(stderr,"%s: must be on local machine to modify ACL.\n",
			ProgramName);
		return 1;
	}

	XmuPrintDefaultErrorMessage(dpy, rep, stderr);
	return 0;
}
